# add_coeffs_to_svg.R

### What the function does ###################################################
# The function adds the membership coefficients of a fuzzy cluster analysis 
# to existing neighbour-net graphs. For compatibility, the graphs should be 
# created with SplitsTree (available from: http://www.splitstree.org/).
# The fuzzy cluster analysis can be carried out with function 'fanny' from 
# the R-package 'cluster' (http://cran.r-project.org/web/packages/cluster/).


### Prerequisits #############################################################
# 1) A matrix containing the membership coefficients of the fuzzy cluster 
# analysis. If you are using the R-package 'cluster', the membership
# coefficients will be in the variable $membership of the 'fanny' object.
# 2) A neighbour-net graph saved as an SVG-file. One way of producing such 
# a file is the following: (¡) write the dissimilarity matrix of your cluster 
# analysis to a file using the write.nex function; open the file with 
# SplitsTrees and save the graph as an SVG file.


### Usage ####################################################################
# 1) Load the function: source("add_coeffs_to_svg.R")
# This assumes that the file is in your current working directory.
# 2) Run the function: add.coeffs.to.svg("svg.file", membership.matrix)
# The arguments 'svg.file' and 'membership.matrix' are, respectively, the path # to the SVG file and the membership matrix (fanny.object$membership).
# The function takes the following optional arguments: 
# 'frequency.vector', containing word frequencies (see Comments below)
# 'fontsize.4', maximum font size for plotting word frequencies
# 'fontsize.1', minimum font size for plotting word frequencies
# 'legend.x' and 'legend.y', positioning of the legend 


### Comments #################################################################
# 1) The script can take up to a maximum of 10 membership coefficients.
# 2) The optional frequency.vector should contain frequencies for the items 
# displayed in the graph. If frequencies are given, the labels will be plotted # in four different font sizes: fontsize.1: <2, fontsize.2: [2,5], fontsize.3: # [5,10], fontsize.4: >10. 
# 3) The SVG are best created using a fixed-with font (e.g. Monaco) with a 
# font size as large as (or slightly larger then) the maximum font size used 
# in the script (default value: 20p).
# 4) The default position of the legend is x=100, y=100.
# 5) As this function directly manipulates SVG files, it may result in 
# overlapping labels (especially for large data sets). These can be avoided 
# by either pre-editing (in SplitsTrees) or post-editing (with a vector 
# graphics editor like Inkscape or Adobe Illustrator) the SVG files.
# 6) If you need to make changes to this function, consult the additional 
# comments in the script below.


### Function #################################################################
# Arguments: svg.file [obligatory], membership.matrix [obligatory], 
# frequency.vector, fontsize.4, fontsize.1, legend.x, legend.y
add.coeffs.to.svg <- function(svg.file, membership.matrix, frequency.vector=NULL, fontsize.4=20, fontsize.1=11, legend.x=100, legend.y=100) {

## Check whether the data is of the correct type
	if (!file.exists(svg.file)) {stop('Please specify an SVG-file in the working directory as the first argument.')}
	if (!is.matrix(membership.matrix)) {stop('Please supply a named MATRIX containing the membership coefficients as the second argument.')}
	if (is.null(rownames(membership.matrix))) {stop('Please make sure that the matrix with the membership coefficients has ROW NAMES.')}

	if (!is.null(frequency.vector)) {
		if (!is.vector(frequency.vector)) {stop('Please supply a VECTOR of frequencies.')}	
		if (is.null(names(frequency.vector))) {stop('Please supply a vector of frequencies with NAMES specifying the word/expression.')}
	}

## Layout variables
# If you want to use different sizes you can specify them directly instead of # using the computed values, e.g. fontsize.2 <- 18
# If you want to use more than just four font sizes to represent word 
# frequencies, you can define additional fontsize variables; you then have to # make corresponding changes to the frequency cut-off points specified below.
	font.name <- "Monaco, monospace"; # We're using a monospaced font
	fontsize.2 <- round(fontsize.1*(fontsize.4/fontsize.1)^(1/3)) 
	fontsize.3 <- round(fontsize.1*((fontsize.4/fontsize.1)^(1/3))^2)
	# With fontsize.1=11 and fontsize.4=20, fontsize.2=13 and fontsize.3=16

## Input and output files
	wd <- paste(getwd(),'/',sep='') # current working directory
	svg.name <- sub('(.*)(\\.svg)', '\\1', svg.file) # filename minus suffix
	svg.in.path <- paste(wd,svg.name,'.svg',sep='')
	svg.in <- file(svg.in.path) # input file
	svg.out.path <- paste(wd,svg.name,'_out','.svg',sep='')
  # Remove earlier output file of the same name
	if (file.exists(svg.out.path)) {
		file.remove(svg.out.path)
	}
	svg.out <- file(svg.out.path) # output file
	svg.content <- readLines(svg.in) # content of input file

## Get horizontal centre of the graph for placement of the labels
# This may require changing depending on the format of the SVG graph
	centre <- as.numeric(sub('(<svg[^>]* width=")([0-9]+)([^>]*>)','\\2',grep('^<svg',svg.content, value=T)))*0.8

## Colours
# Ideally we would be using HSL or HSLA (hue.d <- 360/(ncol(membership)+1)), 
# but these colour modes are not widely supported. We therefore use a vector 
# of 10 preselected colour names. 
	rect.colors <- c("crimson", "green", "mediumblue", "magenta", "gold", "cyan", "darkorange", "mediumseagreen", "deepskyblue", "sienna")

## Find labels and make them identical to the ones in the SVG file
# This replace '_' by '.'. 
# NB. These replacements may not be necessary in your data
	labels.orig <- rownames(membership.matrix)
	labels <- gsub('[_\\(\\)]','\\.',labels.orig)
	rownames(membership.matrix) <- labels
	if (!is.null(frequency.vector)) { names(frequency.vector) <- labels }
	names(labels.orig) <- labels # save the original labels for printing


## Print output file
	open(svg.out, open="w")
	for (l in svg.content) {
# Get the lines from svg file where the label is in 'labels'
		if(grepl('<text',l)) {
			label <- sub('([^>]*>)([^<]+)([^>]+>)','\\2',l)
			if(label %in% labels) {
# Get the current location of the label
				x <- as.numeric(sub('([^>]* x=")([0-9]+)(.+>)','\\2',l))
				y <- as.numeric(sub('([^>]* y=")([0-9]+)(.+>)','\\2',l))

## Frequency cut-off points: assign font sizes to frequencies
# Change the numbers if you need different intervals. Default:
# freq <= 2: fontsize.1
# freq > 2 and <= 5: fontsize.2
# freq > 5 and <= 10: fontsize.3
# freq > 10: fontsize.4
# If you want to use more than four font sizes, you need to add one additional # 'else if' statement for each additional cut-off point.
				if (!is.null(frequency.vector)) {
					if(frequency.vector[label] <= 2) font.size <- fontsize.1
					else if(frequency.vector[label] > 2 & frequency.vector[label] <= 5) font.size <- fontsize.2
					else if(frequency.vector[label] > 5 & frequency.vector[label] <= 10) font.size <- fontsize.3
					else font.size <- fontsize.4
				}
# If no frequency vector is proviede, we use fontsize.2 for all labels
				else {
					font.size <- fontsize.2
				}			

## Layout variables depending on font size
# These have proved appropriate for the default font sizes; they may need 
# changing when very different font sizes are used.
				font.width <- font.size/1.6 # May need adjustments
				rect.width <- font.size-4 # Size of rectangle
				text.offset <- font.size-2 # Offset for text-box
				label.width <- nchar(label)*font.width

## Labels on left of centre: stripe to left of label
				if(x < centre) {

					# Move x-position to the right to compensate for smaller font sizes
					if(font.size==fontsize.4) font.size.offset <- 0
					else {
						font.width.up <- fontsize.4/1.4
						label.width.up <- nchar(label)*font.width.up
						font.size.offset <- label.width.up - label.width
					}			
					# Move x-position to the left by the width of the strip
					stripe.width <- (ncol(membership.matrix)+1)*rect.width
					# Changed x-position
					x.moved <- x-stripe.width*1.0+font.size.offset

					cat('<svg x="', x.moved, '" y="', y-text.offset, '" xml:space="preserve">', sep='', file=svg.out, append=T, fill=T)
					cat('<text y="', text.offset, '" x="', stripe.width, '" xml:space="preserve" style="fill:black; stroke:none; font-family:', font.name, '; font-size:', font.size,'; font-weight:normal">', labels.orig[label], '</text>', sep='', file=svg.out, append=T, fill=T)
					for (n in 1:ncol(membership.matrix)) {
						p <- round(membership.matrix[label,n]^24,3)
						cat('<rect x="', n*rect.width-rect.width/2, '" y="', text.offset-rect.width, '" height="', rect.width, '" width="', rect.width,'" style="stroke:black; stroke-width:0.25; fill:', rect.colors[n], '; fill-opacity:', p, '"/>', sep='', file=svg.out, append=T, fill=T)
					}
				}

## Labels on the right of the centre: stripe to right of label
				else {
					cat('<svg x="', x-text.offset/2, '" y="', y-text.offset, '" xml:space="preserve">', sep='', file=svg.out, append=T, fill=T)
					cat('<text y="', text.offset, '" xml:space="preserve" style="fill:black; stroke:none; font-family:', font.name, '; font-size:', font.size,'; font-weight:normal">', labels.orig[label], '</text>', sep='', file=svg.out, append=T, fill=T)
					for (n in 1:ncol(membership.matrix)) {
						p <- round(membership.matrix[label,n]^24,3)
						cat('<rect x="', label.width+n*rect.width-rect.width/2, '" y="', text.offset-rect.width, '" height="', rect.width, '" width="', rect.width,'" style="stroke:black; stroke-width:0.25; fill:', rect.colors[n], '; fill-opacity:', p, '"/>', sep='', file=svg.out, append=T, fill=T)
					}
				}
				cat('</svg>', sep='', file=svg.out, append=T, fill=T)
			}
		}
		
## Print legend
# We're assuming that the input file from SplitsTrees contains only one <svg> 
# element, and we're adding the code for the legend just before the end-tag.
		else if ((grepl('</svg>',l))) {
			# Define gradiants
			cat('<defs>', file=svg.out, append=T, fill=T)
			for (n in 1:ncol(membership.matrix)) {
				cat('<linearGradient id="', rect.colors[n] ,'" x1="0%" y1="100%" x2="0%" y2="0%">', sep='', file=svg.out, append=T, fill=T)
				cat('<stop offset="0%" style="stop-color:', rect.colors[n] ,'; stop-opacity:0" />', sep='', file=svg.out, append=T, fill=T)
				cat('<stop offset="100%" style="stop-color:', rect.colors[n] ,'; stop-opacity:1" />', sep='', file=svg.out, append=T, fill=T)
				cat('</linearGradient>\n', file=svg.out, append=T, fill=T)
			}
			cat('</defs>', file=svg.out, append=T, fill=T)

			# Position rectangles
			rect.width <- fontsize.4-4 # Size of rectangle based on largest font
			rect.height <- rect.width*3

			for (n in 1:ncol(membership.matrix)) {
				cat('<text x="', legend.x+n*rect.width+rect.width/3,'" y="', legend.y-rect.width/3, '" style="fill:black; stroke:none; font-family:', font.name, '; font-size:', fontsize.3,'; font-weight:normal">', n, '</text>', sep='', file=svg.out, append=T, fill=T)
				cat('<rect x="', legend.x+n*rect.width,'" y="', legend.y, '" width="', rect.width, '" height="',rect.height , '"  stroke="black" stroke-width="0.25" fill="url(#', rect.colors[n],')" />', sep='', file=svg.out, append=T, fill=T)
			}

			# Print svg end tag
			cat('</svg>', file=svg.out, append=T, fill=T)
		}
		else {
			cat(l, file=svg.out, append=T, fill=T)
		}
	}
	close(svg.out)
}
